const UpdaterService = require('../service/UpdaterService');
const serverConfig = require('../../../config/ServerConfig');

class UpdaterControllerEx {
  constructor() {};      

  async installVersion(req, res) {
    const data = req.body;

    const response = await UpdaterService.insertEventTable(
      data.CDFILIAL,
      data.CDOPERADOR,
      data.DEVOPSID,
      data.PRODFILIID,
      data.CURRENTVERSION,
      data.INSTALLVERSION,
      data.CREATENEWVERSION,
      'INSTALACAO_PENDENTE'
    );
    
    res.send(response);
  }

  async removeVersion(req, res) {
    const data = req.body;

    const response = await UpdaterService.insertEventTable(
      data.CDFILIAL,
      data.CDOPERADOR,
      data.DEVOPSID,
      data.PRODFILIID,
      data.CURRENTVERSION,
      data.INSTALLVERSION,
      data.CREATENEWVERSION,
      'REMOCAO_PENDENTE'
    );
    
    res.send(response);
  }

  async installVersionInProgress(req, res) {
    const data = req.body;

    const response = await UpdaterService.updateEventTable(
      data.CDFILIAL,
      data.CDOPERADOR,
      data.DEVOPSID,
      data.CURRENTVERSION,
      data.INSTALLVERSION,
      data.CREATENEWVERSION,
      'INSTALACAO_EM_CURSO'
    );
    
    res.send(response);
  }

  async removeVersionInProgress(req, res) {
    const data = req.body;

    const response = await UpdaterService.updateEventTable(
      data.CDFILIAL,
      data.CDOPERADOR,
      data.DEVOPSID,
      data.CURRENTVERSION,
      data.INSTALLVERSION,
      data.CREATENEWVERSION,
      'REMOCAO_EM_CURSO'
    );
    
    res.send(response);
  }

  async finishUpdater(req, res) {
    const data = req.body;

    const response = await UpdaterService.finishEventTable(
      data.CDFILIAL,
      data.CDOPERADOR,
      data.DEVOPSID,
      data.PRODFILIID,
      data.CURRENTVERSION,
      data.INSTALLVERSION,
      data.CREATENEWVERSION
    );

    res.send(response);
  }

  async cancelUpdater(req, res) {
    const data = req.body;

    const response = await UpdaterService.cancelEventTable(
      data.CDFILIAL,
      data.CDOPERADOR,
      data.DEVOPSID,
      data.CURRENTVERSION,
      data.INSTALLVERSION,
      data.CREATENEWVERSION
    );
    
    res.send(response);
  }
  
  async cancelUnitEvents(req, res) {
    const data = req.body;

    await UpdaterService.cancelUnitEvents(
      data.CDFILIAL
    );

    const response = {
      error: false,
      message: `Os eventos da filial ${data.CDFILIAL} foram cancelados.`
    }
        
    res.send(response);
  }

  async getPendingEvents(req, res) {
    const pendingEvents = await UpdaterService.getPendingEvents();
    
    const response = {
      error: true,
      message: '',
      data: {}
    };

    if (pendingEvents.rows.length) {
      response.error = false;
      response.message = '';
      response.data = pendingEvents.rows;
    }

    res.send(response);
  }

  async getPendingEventsByUnit(req, res) {
    const data = req.body;

    const response = await UpdaterService.getPendingEventsByUnit(data.CDFILIAL);
    
    res.send(response);
  }

  async getAvailablePackages(req, res) {
    const data = req.body;
    const extension = 'tar.gz';
    
    const packageURL = `${serverConfig.packageServer}/${data.DIRECTORY}/${data.INSTALLVERSION}.${extension}`

    res.send({url: packageURL});
  }
  
  async checkInstalledVersion(req, res){
    const params = req.body   
    const checkVersionInstall = await UpdaterService.checkVersion(params)  
    res.send(checkVersionInstall)
  }

  async saveConfigDb(req, res){
    const params = req.body;
  
    const response = await UpdaterService.saveConfig(params);
    
    res.send(response);
  }

  async getConfigIniByUnit(req, res) {
    const params = req.body;

    const response = await UpdaterService.getConfigIniByUnit(params.CDFILIAL);

    res.send(response);
  }

  async saveConfigIniByUnit(req, res) {
    const params = req.body;

    const response = await UpdaterService.saveConfigIniByUnit(
      params.CDFILIAL,
      params.CDOPERADOR,
      params.CONFIG
    );

    res.send(response);
  }

  async getConfigHistoryByUnit(req, res) {
    const params = req.body;

    const response = await UpdaterService.getConfigHistoryByUnit(params.CDFILIAL);

    res.send(response);
  }

  async getProdConfig(req, res){
    const params = req.body;
   
    const response = await UpdaterService.getProdConfig(params.UPDATERPRODID, params.NRVERSAOATUAL);

    res.send(response);
  }

  async saveProdConfig(req, res){
    const params = req.body;

    const response = await UpdaterService.saveProdConfig(params.CDPRODUCTID, params.NRVERSAO, params.CONFIG);
    
    res.send(response);
  }

  async startServiceMonitor(req, res) {
    const params = req.body;

    const services = params.map(function(service){
      return service.serviceName
    });

    const response = await UpdaterService.startServiceMonitor(services);

    res.send(response);
  }

  async stopServiceMonitor(req, res) {
    const params = req.body;

    const response = await UpdaterService.stopServiceMonitor(params);
    
    res.send(response);
  }
}

module.exports = new UpdaterControllerEx()