import { AlertService, LoadingService, Modal, ModalService, Button, Select, DialogService, Form, Text } from '@zeedhi/common';
import { Config, Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';
import { SocketEvent } from '@/socket/socket-event';

export class UpdatesController {
  public socket?: Socket;

  private fullscreenModal!: Modal;

  private logModal!: Modal;

  private prodConfigModal!: Modal;

  private prodConfigEditModal!: Modal;

  private defaultGridTitle!: String;

  private filterProductName!: String;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  private logRow!: any;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('historico', true);
    this.logModal = await ModalService.createFromJson('updater-log-filial', true);
    this.prodConfigModal = await ModalService.createFromJson('updater-prod-config-modal', true);
    this.prodConfigEditModal = await ModalService.createFromJson('updater-prod-config-edit-modal', true);
  }

  private verifyAndShowUnitStatus(row : any) {
    if (!row.CONNECTION_STATUS) this.showAlert('A conexão com a unidade deve estar ativa para realizar esta ação', '#e68129');
    return row.CONNECTION_STATUS;
  }

  public async openLogBranchModal({ row }: any) {
    if (!this.verifyAndShowUnitStatus(row)) return;
    const params = { CDFILIAL: row.CDFILIAL };
    this.logRow = row;
    this.logModal.show();

    if (params) {
      this.socket?.emit('getLogUpdaterBranch', params);
    }
  }

  public updaterLogFormMounted(params: any) {
    const { component: form } = params;
    const divTextArea: any = form.getChildInstance('updater-text-area-result');
    const textArea: any = document.getElementById('updater-text-area-result');
    const textFooter: Text = form.getChildInstance('updater-branch-log');
    textFooter.text += ` ${this.logRow.CDFILIAL}`;
    this.socket?.removeAllListeners('logUpdaterBranchFinish');
    this.socket?.on('logUpdaterBranchFinish', (param) => {
      if (param.error === false) {
        divTextArea.value += `${param.message}\n`;
        textArea.scrollTop = textArea.scrollHeight;
      }
    });
  }

  public openFullscreenModal({ row }: any) {
    LoadingService.show();

    const filter = {
      params: {
        CDFILIAL: row.CDFILIAL,
        CDPRODUCTID: row.ID,
      },
    };

    Http.get('/updaterProdutoHistorico', filter)
      .then((result) => {
        const grid = this.fullscreenModal.children[1];
        grid.datasource.data = result.data.data;
        this.fullscreenModal.show();
      })
      .catch((error) => {
        this.showAlert(error, 'error');
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  public async gridMounted(force: boolean = false) {
    if (!this.socket) {
      this.socket = io(Config.endPoint);
    }

    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socketListener();

    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const date = new Date().toLocaleDateString();
    const savedDate = localStorage.getItem('LAST_DATE_UPDATER');
    this.defaultGridTitle = grid.title;

    if (savedDate !== date || (typeof force === 'boolean' && force)) {
      LoadingService.show('Atualizando lista de sistemas...');
      const conexao = `${Config.metadataEndPoint}orgProducts`;
      Http.get(conexao)
        .then((result) => {
          if (result.data.isError) {
            this.showAlert(`Erro ao atualizar a lista de sistemas: ${result.data.response}`, 'error');
          } else {
            this.showAlert('Lista de sistemas atualizada com sucesso!', 'success');
          }
        })
        .catch((error) => {
          this.showAlert(error, 'error');
        })
        .finally(() => {
          LoadingService.hide();
          localStorage.setItem('LAST_DATE_UPDATER', date);
          if (typeof force !== 'boolean') {
            this.openFilter(grid);
          }
        });
    } else {
      this.openFilter(grid);
    }
  }

  private openFilter(grid: TekGrid) {
    const filterButton = Metadata.getInstance<Button>(`${grid.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public updateSystemList(): void {
    this.gridMounted(true);
  }

  private showAlert(message: any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  public executeDelete(row: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');

    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      PRODFILIID: row.PRODFILIID.toString(),
      DEVOPSID: row.ID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL,
      INSTALLVERSION: null,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'REMOVE',
    };

    Http.post('/updater/removeVersion', params).then((result) => {
      if (result.data && result.data.error) {
        this.showAlert(result.data.message, 'error');
      } else {
        this.socket?.emit('execUpdater', params);
        this.showAlert('Requisição de remoção enviada com sucesso!', 'success');
        grid.datasource.get();
      }
    })
      .catch((error) => {
        this.showAlert(error, 'error');
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public deleteVersion({ row }: any) {
    try {
      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.IDSTEVENTO === 'INSTALACAO_PENDENTE') {
        throw new Error('Sistema não instalado.');
      }

      DialogService.show({
        name: 'remove-dialog',
        type: 'warning',
        title: 'Exclusão',
        text: 'Deseja realmente excluir essa versão desse sistema?',
        buttons: [
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                DialogService.hide();
                this.executeDelete(row);
              },
            },
          },
        ],
      });
    } catch (error) {
      if (error instanceof Error) {
        this.showAlert(error.message, '#e68129');
      }
    }
  }

  private async executeInstall(row: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');

    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      PRODFILIID: row.PRODFILIID.toString(),
      DEVOPSID: row.ID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL === '-' ? null : row.NRVERSAOATUAL,
      INSTALLVERSION: row.VERSION.NRVERSAO,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'INSTALL',
    };

    Http.post('/updater/installVersion', params)
      .then((result) => {
        if (result.data && result.data.error) {
          this.showAlert(result.data.message, 'error');
        } else {
          this.socket?.emit('execUpdater', params);
          this.showAlert('Requisição de instalação/atualização enviada com sucesso!', 'success');
          grid.updateGroupedData(false);
        }
      })
      .catch((error) => {
        this.showAlert(error, 'error');
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public async installVersion({ row }: any) {
    try {
      if (!row.VERSION) {
        throw new Error('Selecione a versão.');
      }

      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.VERSION.IDDOWNLOADVER === 'N') {
        throw new Error('Essa versão ainda não está disponível para download. Aguarde alguns instantes e tente novamente.');
      }

      if (row.VERSION.IDDOWNLOADVER === 'E') {
        throw new Error('Ocorreu um erro ao realizar o download dessa versão para a nuvem. Atualize a lista de sistemas, aguarde alguns instantes e tente novamente.');
      }

      const params = {
        CDFILIAL: row.CDFILIAL,
        UPDATERPRODID: row.ID,
        NRVERSAOATUAL: row.VERSION.NRVERSAO,
      };

      const checkVersionInstalled = await Http.post('/updater/checkVersion', params);

      if (checkVersionInstalled.data && checkVersionInstalled.data.error) {
        throw new Error('Essa versão já está instalada.');
      }

      const { data } = await Http.post('/updater/getProdConfig', params);

      if (data && data.error && data.message) {
        throw new Error(data.message);
      }

      if (data && data.message) {
        this.showAlert(data.message, '#e68129');
        await this.validateGetUpdaterProdConfig(data, row);
      } else {
        this.dialog(data.data, row);
      }
    } catch (error) {
      if (error instanceof Error) {
        this.showAlert(error, '#e68129');
      }
    }
  }

  public async validateGetUpdaterProdConfig(data: any, row: any) {
    const footerModal = this.prodConfigModal.children.find((el: any) => el.name === 'updater-prod-config-modal-footer');
    const buttonSaveModal = footerModal?.rightSlot.find((el:any) => el.name === 'save-button-prod-config');
    if (buttonSaveModal && buttonSaveModal.events) {
      buttonSaveModal.events.click = () => {
        this.saveUpdaterProdConfig(row);
      };
    }
    await this.prodConfigModal.show();
    const form: any = Metadata.getInstance<Form>('updater-prod-config-form');
    const iterable: any = form.getChildInstance('IterablecomponentParent');

    const formInputNrVersao: any = this.prodConfigModal.getChildInstance('NRVERSAO');
    formInputNrVersao.value = row.VERSION?.NRVERSAO;

    const formInputCdProductId: any = this.prodConfigModal.getChildInstance('CDPRODUCTID');
    formInputCdProductId.value = row.ID;

    iterable.datasource.data = [];

    Object.entries(data.data).forEach(([key, element]) => {
      const productConfigObject = {
        id: key,
        value: element,
        label: key,
      };
      if (productConfigObject) {
        iterable.datasource.data.push(productConfigObject);
      }
    });
  }

  public async saveUpdaterProdConfig(rowGrid: any) {
    try {
      const form: any = Metadata.getInstance<Form>('updater-prod-config-form');
      const nrVersao: any = this.prodConfigModal.getChildInstance('NRVERSAO');
      const cdProductId: any = this.prodConfigModal.getChildInstance('CDPRODUCTID');
      const iterable: any = form.getChildInstance('IterablecomponentParent');
      const inputs = [...form.element].filter((item) => item.type === 'text');
      const obj: any = {};

      if (inputs && iterable) {
        inputs.forEach((row: any) => {
          iterable.datasource.data.forEach((item: any) => {
            if (row.id === item.id) {
              obj[item.label] = row.value;
            }
          });
        });
      }

      const params = {
        CDPRODUCTID: cdProductId.value,
        NRVERSAO: nrVersao.value,
        CONFIG: obj,
      };
      const { data } = await Http.post('/updater/saveProdConfig', params);

      if (data && data.error) {
        throw new Error(data.message);
      }

      this.showAlert(data.message, 'success');
      this.prodConfigModal.hide();
      this.dialog(data.data, rowGrid);
    } catch (error) {
      this.showAlert(error, '#e68129');
    }
  }

  public async createProdConfigEditModal({ row }: any) {
    try {
      if (!row.VERSION) {
        throw new Error('Selecione a versão.');
      }

      const params = {
        UPDATERPRODID: row.ID,
        NRVERSAOATUAL: row.VERSION.NRVERSAO,
      };

      const { data } = await Http.post('/updater/getProdConfig', params);

      if (data && data.error && data.message) {
        throw new Error(data.message);
      }

      if (data && data.message) {
        this.showAlert(data.message, '#e68129');
        await this.validateGetUpdaterProdConfig(data, row);
      } else {
        await this.prodConfigEditModal.show();
        const form: any = Metadata.getInstance<Form>('updater-prod-config-edit-form');
        const iterable: any = form.getChildInstance('IterablecomponentParent');

        iterable.datasource.data = [];
        const formInputNrVersaoEdit: any = this.prodConfigEditModal.getChildInstance('NRVERSAO');
        formInputNrVersaoEdit.value = row.VERSION?.NRVERSAO;
        const formInputCdProductIdEdit: any = this.prodConfigEditModal.getChildInstance('CDPRODUCTID');
        formInputCdProductIdEdit.value = row.ID;

        Object.entries(data.data).forEach(([key, element]) => {
          const productConfigParametersObject = {
            id: `IterablecomponentParent-${key}`,
            value: element,
            label: key,
          };
          if (productConfigParametersObject) {
            iterable.datasource.data.push(productConfigParametersObject);
          }
        });
      }
    } catch (error) {
      this.showAlert(error, '#e68129');
    }
  }

  public async saveUpdaterProdConfigEdit() {
    try {
      const form: any = Metadata.getInstance<Form>('updater-prod-config-edit-form');
      const iterable: any = form.getChildInstance('IterablecomponentParent');
      const inputs = [...form.element].filter((item) => item.type === 'text');

      const formInputNrVersaoEdit: any = this.prodConfigEditModal.getChildInstance('NRVERSAO');
      const formInputCdProductIdEdit: any = this.prodConfigEditModal.getChildInstance('CDPRODUCTID');
      const obj: any = {};

      if (inputs && iterable) {
        inputs.forEach((row: any) => {
          iterable.datasource.data.forEach((item: any) => {
            if (row.id === item.id) {
              obj[item.label] = row.value;
            }
          });
        });
      }

      const params = {
        CDPRODUCTID: formInputCdProductIdEdit.value,
        NRVERSAO: formInputNrVersaoEdit.value,
        CONFIG: obj,
      };

      const { data } = await Http.post('/updater/saveProdConfig', params);

      if (data && data.error) {
        throw new Error(data.message);
      }

      this.showAlert(data.message, 'success');
      this.closeUpdaterProdConfigEdit();
    } catch (error) {
      this.showAlert(error, '#e68129');
    }
  }

  private dialog(data: any, row?: any): void {
    let prodConfigValues = '';

    Object.entries(data).forEach(([key, value]) => {
      prodConfigValues += `<strong>${key}: ${value}</strong><br>`;
    });
    const hasParameters = !!prodConfigValues;
    const textDialogPrefix = hasParameters ? `<p>Parâmetros do Projeto</p>\n${prodConfigValues}` : '<strong>Projeto não possui parâmetros</strong>';
    DialogService.show({
      name: 'install-dialog',
      type: 'warning',
      title: 'Instalação',
      text: `    
      ${textDialogPrefix}
      <br>Deseja realmente instalar/atualizar esse sistema?                      
        `,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              this.executeInstall(row);
            },
          },
        },
      ],
    });
  }

  private async socketListener() {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    this.socket?.on('updaterInProgress', (result) => {
      let message = 'Instalação iniciada...';
      if (result === 'UPDATE') {
        message = 'Atualização iniciada...';
      } else if (result === 'REMOVE') {
        message = 'Remoção iniciada...';
      }
      this.showAlert(message, 'success');
      grid.datasource.get();
    });

    this.socket?.on('updaterFinish', (result) => {
      if (result === 'ERROR') {
        this.showAlert('Ocorreu um erro durante a execução do último processo. Verifique o histórico para mais informações.', 'error');
      } else {
        this.showAlert('Operação finalizada com sucesso!', 'success');
      }
      grid.datasource.get();
    });
    this.socket?.on('disconnect', () => {
      this.showAlert('A página será recarregada para restabelecer a conexão...', 'error');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });

    this.socket?.on('updaterConnected', async () => {
      grid.datasource.get();
    });
  }

  public setAvailableVersionFilter({ component }: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    component.datasource.filter = grid.datasource.filter;
  }

  public beforeApplyFilter({ component }: any) {
    // Force to reload all available version column
    component.datasource.updateData([]);
    component.toolbarSlot[0].text = `${this.defaultGridTitle} - ${this.filterProductName}`;
  }

  public changeProductId({ component, value }: any) {
    const data = component.datasource.data.find((item: any) => item.ID === value);
    if (data) {
      this.filterProductName = data.NMPRODUCT;
    }
  }

  public gridBeforeDestroy() {
    this.socket?.removeAllListeners('updaterInProgress');
    this.socket?.removeAllListeners('updaterFinish');
    this.socket?.removeAllListeners('updaterConnected');
    this.socket?.removeAllListeners('disconnect');
    this.fullscreenModal?.destroy();
    this.logModal?.destroy();
    this.prodConfigModal.destroy();
    this.prodConfigEditModal.destroy();
  }

  public closeProdConfigModal(): void {
    this.prodConfigModal.hide();
  }

  public closeUpdaterProdConfigEdit(): void {
    this.prodConfigEditModal.hide();
  }

  public getConnectionColor({ row }: any) {
    return row.CONNECTION_STATUS ? 'green' : 'red';
  }

  public changeSelectVersion({ value }: IEventParam<Select>) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    grid.datasource.currentRow.VERSION = value;
  }
}
